-- luacheck: globals app os verboseLevel connect
local app = app
local Class = require "Base.Class"
local Unit = require "Unit"
local GainBias = require "Unit.ViewControl.GainBias"
local OutputScope = require "Unit.ViewControl.OutputScope"
local Encoder = require "Encoder"
local ply = app.SECTION_PLY

local FoldUnit = Class{}
FoldUnit:include(Unit)

function FoldUnit:init(args)
  args.title = "Fold"
  args.mnemonic = "F"
  Unit.init(self,args)
end

function FoldUnit:onLoadGraph(channelCount)
  if channelCount==2 then
    self:loadStereoGraph()
  else
    self:loadMonoGraph()
  end
end

function FoldUnit:loadMonoGraph()
  local fold = self:createObject("Fold","fold")

  local upper = self:createObject("GainBias","upper")
  local upperRange = self:createObject("MinMax","upperRange")
  local lower = self:createObject("GainBias","lower")
  local lowerRange = self:createObject("MinMax","lowerRange")

  local threshold = self:createObject("GainBias","threshold")
  local thresholdRange = self:createObject("MinMax","thresholdRange")

  connect(threshold,"Out",thresholdRange,"In")
  connect(threshold,"Out",fold,"Threshold")
  connect(upper,"Out",upperRange,"In")
  connect(upper,"Out",fold,"Upper Gain")
  connect(lower,"Out",lowerRange,"In")
  connect(lower,"Out",fold,"Lower Gain")

  connect(self,"In1",fold,"In")
  connect(fold,"Out",self,"Out1")

  self:createMonoBranch("threshold",threshold,"In",threshold,"Out")
  self:createMonoBranch("upper",upper,"In",upper,"Out")
  self:createMonoBranch("lower",lower,"In",lower,"Out")
end

function FoldUnit:loadStereoGraph()
  local fold1 = self:createObject("Fold","fold1")
  local fold2 = self:createObject("Fold","fold2")

  local upper = self:createObject("GainBias","upper")
  local upperRange = self:createObject("MinMax","upperRange")
  local lower = self:createObject("GainBias","lower")
  local lowerRange = self:createObject("MinMax","lowerRange")

  local threshold = self:createObject("GainBias","threshold")
  local thresholdRange = self:createObject("MinMax","thresholdRange")

  connect(threshold,"Out",thresholdRange,"In")
  connect(threshold,"Out",fold1,"Threshold")
  connect(threshold,"Out",fold2,"Threshold")
  connect(upper,"Out",upperRange,"In")
  connect(upper,"Out",fold1,"Upper Gain")
  connect(upper,"Out",fold2,"Upper Gain")
  connect(lower,"Out",lowerRange,"In")
  connect(lower,"Out",fold1,"Lower Gain")
  connect(lower,"Out",fold2,"Lower Gain")

  connect(self,"In1",fold1,"In")
  connect(fold1,"Out",self,"Out1")
  connect(self,"In2",fold2,"In")
  connect(fold2,"Out",self,"Out2")

  self:createMonoBranch("threshold",threshold,"In",threshold,"Out")
  self:createMonoBranch("upper",upper,"In",upper,"Out")
  self:createMonoBranch("lower",lower,"In",lower,"Out")
end

local views = {
  expanded = {"threshold","upper","lower"},
  collapsed = {},
  threshold = {"scope","threshold"},
  upper = {"scope","upper"},
  lower = {"scope","lower"},
}

function FoldUnit:onLoadViews(objects,branches)
  local controls = {}

  controls.scope = OutputScope {
    monitor = self,
    width = 4*ply,
  }

  controls.threshold = GainBias {
    button = "thresh",
    branch = branches.threshold,
    description = "Threshold",
    gainbias = objects.threshold,
    range = objects.thresholdRange,
    biasMap = Encoder.getMap("[-1,1]"),
    biasUnits = app.unitNone,
    initialBias = 0.0,
    gainMap = Encoder.getMap("gain"),
  }

  controls.upper = GainBias {
    button = "upper",
    branch = branches.upper,
    description = "Upper Gain",
    gainbias = objects.upper,
    range = objects.upperRange,
    biasMap = Encoder.getMap("[-5,5]"),
    biasUnits = app.unitNone,
    initialBias = 1.0,
    gainMap = Encoder.getMap("gain"),
  }

  controls.lower = GainBias {
    button = "lower",
    branch = branches.lower,
    description = "Lower Gain",
    gainbias = objects.lower,
    range = objects.lowerRange,
    biasMap = Encoder.getMap("[-5,5]"),
    biasUnits = app.unitNone,
    initialBias = 1.0,
    gainMap = Encoder.getMap("gain"),
  }

  return controls, views
end

return FoldUnit
