-- luacheck: globals app os verboseLevel connect tie
local app = app
local Class = require "Base.Class"
local Unit = require "Unit"
local GainBias = require "Unit.ViewControl.GainBias"
local Encoder = require "Encoder"
local ply = app.SECTION_PLY

local Velvet = Class{}
Velvet:include(Unit)

function Velvet:init(args)
  args.title = "Velvet Noise"
  args.mnemonic = "VN"
  Unit.init(self,args)
end

function Velvet:onLoadGraph(channelCount)
  if channelCount==2 then
    self:loadStereoGraph()
  else
    self:loadMonoGraph()
  end

  local rate = self:createObject("ParameterAdapter","rate")
  tie(self.objects.noise1,"Rate",rate,"Out")

  self:createMonoBranch("rate",rate,"In",rate,"Out")
end

function Velvet:loadMonoGraph()
  local noise1 = self:createObject("VelvetNoise","noise1")
  connect(noise1,"Out",self,"Out1")
end

function Velvet:loadStereoGraph()
  local noise1 = self:createObject("VelvetNoise","noise1")
  local noise2 = self:createObject("VelvetNoise","noise2")
  connect(noise1,"Out",self,"Out1")
  connect(noise2,"Out",self,"Out2")
  tie(noise2,"Rate",noise1,"Rate")
end

local views = {
  expanded = {"rate"},
  collapsed = {},
}

function Velvet:onLoadViews(objects,branches)
  local controls = {}

  controls.rate = GainBias {
    button = "rate",
    description = "Impulse Rate",
    branch = branches.rate,
    gainbias = objects.rate,
    range = objects.rate,
    biasMap = Encoder.getMap("oscFreq"),
    biasUnits = app.unitHertz,
    initialBias = 880,
    gainMap = Encoder.getMap("freqGain"),
    scaling = app.octaveScaling
  }
  
  return controls, views
end

return Velvet
