-- luacheck: globals app os verboseLevel connect tie
local app = app
local Class = require "Base.Class"
local Unit = require "Unit"
local Fader = require "Unit.ViewControl.Fader"
local GainBias = require "Unit.ViewControl.GainBias"
local Task = require "Unit.MenuControl.Task"
local MenuHeader = require "Unit.MenuControl.Header"
local ModeSelect = require "Unit.MenuControl.OptionControl"
local Encoder = require "Encoder"
local Utils = require "Utils"
local ply = app.SECTION_PLY

local SchroederAllPass = Class{}
SchroederAllPass:include(Unit)

function SchroederAllPass:init(args)
  args.title = "Schroeder Allpass"
  args.mnemonic = "SA"
  Unit.init(self,args)
end

function SchroederAllPass:onLoadGraph(channelCount)
  local delay = self:createObject("Delay","delay",1)
  delay:allocateTimeUpTo(0.05)
  local delayAdapter = self:createObject("ParameterAdapter","delayAdapter")
  local gainAdapter = self:createObject("ParameterAdapter","gainAdapter")
  local feedBackMix = self:createObject("Sum","feedBackMix")
  local feedBackGain = self:createObject("ConstantGain","feedBackGain")
  local feedForwardMix = self:createObject("Sum","feedForwardMix")
  local feedForwardGain = self:createObject("ConstantGain","feedForwardGain")
  local limiter = self:createObject("Limiter","limiter")

  connect(self,"In1",feedBackMix,"Left")
  connect(feedBackMix,"Out",delay,"Left In")
  connect(delay,"Left Out",feedForwardMix,"Left")
  connect(feedForwardMix,"Out",self,"Out1")
  connect(feedBackMix,"Out",feedForwardGain,"In")
  connect(feedForwardGain,"Out",feedForwardMix,"Right")
  connect(delay,"Left Out",feedBackGain,"In")
  connect(feedBackGain,"Out",limiter,"In")
  connect(limiter,"Out",feedBackMix,"Right")

  tie(delay,"Left Delay",delayAdapter,"Out")
  tie(feedBackGain,"Gain",gainAdapter,"Out")
  tie(feedForwardGain,"Gain","negate",gainAdapter,"Out")

  self:createMonoBranch("delay",delayAdapter,"In",delayAdapter,"Out")
  self:createMonoBranch("gain",gainAdapter,"In",gainAdapter,"Out")
end

local views = {
  expanded = {"delay","gain"},
  collapsed = {},
}

local function linMap(min,max,n)
  local map = app.LinearDialMap(min,max)
  map:setCoarseRadix(n)
  return map
end

local delayMap = linMap(0,0.05,100)

function SchroederAllPass:onLoadViews(objects,branches)
  local controls = {}

  controls.delay = GainBias {
    button = "delay",
    description = "Delay Time",
    branch = branches.delay,
    gainbias = objects.delayAdapter,
    range = objects.delayAdapter,
    biasMap = delayMap,
    biasUnits = app.unitSecs,
    initialBias = 0.005
  }

  controls.gain = GainBias {
    button = "gain",
    description = "Loop Gain",
    branch = branches.gain,
    gainbias = objects.gainAdapter,
    range = objects.gainAdapter,
    biasMap = nil,
    biasUnits = app.unitNone,
    initialBias = 0.7
  }

  return controls, views
end

function SchroederAllPass:onRemove()
  self.objects.delay:deallocate()
  Unit.onRemove(self)
end

return SchroederAllPass
