-- luacheck: globals app os verboseLevel connect tie
local app = app
local Class = require "Base.Class"
local Unit = require "Unit"
local GainBias = require "Unit.ViewControl.GainBias"
local Encoder = require "Encoder"
local ply = app.SECTION_PLY

local StressUnit = Class{}
StressUnit:include(Unit)

function StressUnit:init(args)
  args.title = "Stress"
  args.mnemonic = "St"
  Unit.init(self,args)
end

function StressUnit:onLoadGraph(channelCount)
  if channelCount==2 then
    self:loadStereoGraph()
  else
    self:loadMonoGraph()
  end
end

function StressUnit:loadMonoGraph()
  local stress = self:createObject("Stress","stress")
  local map = self:createObject("ParameterAdapter","map")
  map:clamp(0,1)

  connect(self,"In1",stress,"In")
  connect(stress,"Out",self,"Out1")
  tie(stress,"Load",map,"Out")

  self:createMonoBranch("load",map,"In",map,"Out")
end

function StressUnit:loadStereoGraph()
  local stress1 = self:createObject("Stress","stress1")
  local stress2 = self:createObject("Stress","stress2")
  local map = self:createObject("ParameterAdapter","map")
  map:clamp(0,1)

  connect(self,"In1",stress1,"In")
  connect(self,"In2",stress2,"In")
  connect(stress1,"Out",self,"Out1")
  connect(stress2,"Out",self,"Out2")
  tie(stress1,"Load",map,"Out")
  tie(stress2,"Load",map,"Out")

  self.objects.stress = self.objects.stress1
  stress1:setAdjustment(0.5)
  stress2:setAdjustment(0.5)

  self:createMonoBranch("load",map,"In",map,"Out")
end

local views = {
  expanded = {"cpu"},
  collapsed = {},
}

function StressUnit:onLoadViews(objects,branches)
  local controls = {}
  controls.cpu = GainBias {
    button = "load",
    description = "CPU Load",
    branch = branches.load,
    gainbias = objects.map,
    range = objects.map,
    biasMap = Encoder.getMap("unit"),
  }
  return controls, views
end

return StressUnit
