-- luacheck: globals app os verboseLevel connect tie
local app = app
local Class = require "Base.Class"
local ClockBase = require "builtins.Timing.ClockBase"
local GainBias = require "Unit.ViewControl.GainBias"
local InputGate = require "Unit.ViewControl.InputGate"
local Encoder = require "Encoder"
local ply = app.SECTION_PLY

local ClockInHertz = Class{}
ClockInHertz:include(ClockBase)

function ClockInHertz:init(args)
  ClockBase.init(self,args)
end

function ClockInHertz:onLoadGraph(channelCount)
  local clock = self:createObject("ClockInHertz","clock")
  ClockBase.loadBaseGraph(self, channelCount, clock)
  local freq = self:createObject("ParameterAdapter","freq")
  tie(clock,"Frequency",freq,"Out")
  self:createMonoBranch("freq",freq,"In",freq,"Out")
end

local views = {
  expanded = {"sync","freq","mult","div","width"},
  collapsed = {},
}

function ClockInHertz:onLoadViews(objects,branches)
  local controls = ClockBase.loadBaseView(self,objects,branches)

  controls.freq = GainBias {
    button = "freq",
    description = "Clock Frequency",
    branch = branches.freq,
    gainbias = objects.freq,
    range = objects.freq,
    biasMap = Encoder.getMap("clockFreq"),
    biasUnits = app.unitHertz,
    scaling = app.octaveScaling,
    initialBias = 2
  }

  return controls, views
end

return ClockInHertz
