-- luacheck: globals app os verboseLevel connect tie
local app = app
local Class = require "Base.Class"
local ClockBase = require "builtins.Timing.ClockBase"
local GainBias = require "Unit.ViewControl.GainBias"
local InputGate = require "Unit.ViewControl.InputGate"
local Encoder = require "Encoder"
local ply = app.SECTION_PLY

local ClockInSeconds = Class{}
ClockInSeconds:include(ClockBase)

function ClockInSeconds:init(args)
  ClockBase.init(self,args)
end

function ClockInSeconds:onLoadGraph(channelCount)
  local clock = self:createObject("ClockInSeconds","clock")
  ClockBase.loadBaseGraph(self, channelCount, clock)
  local period = self:createObject("ParameterAdapter","period")
  tie(clock,"Period",period,"Out")
  self:createMonoBranch("period",period,"In",period,"Out")
end

local views = {
  expanded = {"sync","period","mult","div","width"},
  collapsed = {},
}

function ClockInSeconds:onLoadViews(objects,branches)
  local controls = ClockBase.loadBaseView(self,objects,branches)

  controls.period = GainBias {
    button = "period",
    description = "Clock Period",
    branch = branches.period,
    gainbias = objects.period,
    range = objects.period,
    biasMap = Encoder.getMap("[0,10]"),
    biasUnits = app.unitSecs,
    initialBias = 0.5
  }

  return controls, views
end

return ClockInSeconds
