-- luacheck: globals app os verboseLevel connect tie
local app = app
local Class = require "Base.Class"
local Unit = require "Unit"
local Fader = require "Unit.ViewControl.Fader"
local InputGate = require "Unit.ViewControl.InputGate"
local GainBias = require "Unit.ViewControl.GainBias"
local Encoder = require "Encoder"
local ply = app.SECTION_PLY

local PulseToSeconds = Class{}
PulseToSeconds:include(Unit)

function PulseToSeconds:init(args)
  args.title = "Pulse to Seconds"
  args.mnemonic = "PS"
  Unit.init(self,args)
end

function PulseToSeconds:onLoadGraph(channelCount)
  if channelCount==2 then
    self:loadStereoGraph()
  else
    self:loadMonoGraph()
  end
end

function PulseToSeconds:loadMonoGraph()
  local tap = self:createObject("TapTempo","tapL")
  local period = self:createObject("Constant","periodL")
  local comparator = self:createObject("Comparator","edgeL")

  connect(self,"In1",comparator,"In")
  connect(comparator,"Out",tap,"In")
  tie(period,"Value",tap,"Derived Period")
  connect(period,"Out",self,"Out1")

  local multiplier = self:createObject("ParameterAdapter","multiplier")
  local divider = self:createObject("ParameterAdapter","divider")
  tie(tap,"Multiplier",multiplier,"Out")
  tie(tap,"Divider",divider,"Out")

  self:createMonoBranch("multiplier",multiplier,"In",multiplier,"Out")
  self:createMonoBranch("divider",divider,"In",divider,"Out")
end

function PulseToSeconds:loadStereoGraph()
  local tapL = self:createObject("TapTempo","tapL")
  local periodL = self:createObject("Constant","periodL")
  local edgeL = self:createObject("Comparator","edgeL")

  local tapR = self:createObject("TapTempo","tapR")
  local periodR = self:createObject("Constant","periodR")
  local edgeR = self:createObject("Comparator","edgeR")

  local multiplier = self:createObject("ParameterAdapter","multiplier")
  local divider = self:createObject("ParameterAdapter","divider")
  tie(tapL,"Multiplier",multiplier,"Out")
  tie(tapL,"Divider",divider,"Out")
  tie(tapR,"Multiplier",multiplier,"Out")
  tie(tapR,"Divider",divider,"Out")

  connect(self,"In1",edgeL,"In")
  connect(edgeL,"Out",tapL,"In")
  tie(periodL,"Value",tapL,"Base Period")
  connect(periodL,"Out",self,"Out1")

  connect(self,"In2",edgeR,"In")
  connect(edgeR,"Out",tapR,"In")
  tie(periodR,"Value",tapR,"Base Period")
  connect(periodR,"Out",self,"Out2")

  self:createMonoBranch("multiplier",multiplier,"In",multiplier,"Out")
  self:createMonoBranch("divider",divider,"In",divider,"Out")
end

local views = {
  expanded = {"input","mult","div"},
  collapsed = {},
}

function PulseToSeconds:onLoadViews(objects,branches)
  local controls = {}

  controls.input = InputGate {
    button = "input",
    description = "Unit Input",
    comparator = objects.edgeL,
    readoutUnits = app.unitSecs
  }

  controls.mult = GainBias {
    button = "mult",
    branch = branches.multiplier,
    description = "Clock Multiplier",
    gainbias = objects.multiplier,
    range = objects.multiplier,
    biasMap = Encoder.getMap("int[1,32]"),
    gainMap = Encoder.getMap("[-20,20]"),
    initialBias = 1,
    biasPrecision = 0
  }

  controls.div = GainBias {
    button = "div",
    branch = branches.divider,
    description = "Clock Divider",
    gainbias = objects.divider,
    range = objects.divider,
    biasMap = Encoder.getMap("int[1,32]"),
    gainMap = Encoder.getMap("[-20,20]"),
    initialBias = 1,
    biasPrecision = 0
  }
  return controls, views
end

return PulseToSeconds
