-- luacheck: globals app os verboseLevel connect tie
local app = app
local Class = require "Base.Class"
local Unit = require "Unit"
local Gate = require "Unit.ViewControl.Gate"
local InputGate = require "Unit.ViewControl.InputGate"
local Encoder = require "Encoder"
local ply = app.SECTION_PLY

local QuantizeToClock = Class{}
QuantizeToClock:include(Unit)

function QuantizeToClock:init(args)
  args.title = "Quantize to Clock"
  args.mnemonic = "QC"
  Unit.init(self,args)
end

function QuantizeToClock:onLoadGraph(channelCount)
  local quantizer = self:createObject("QuantizeToClock","quantizer")
  local clockEdge = self:createObject("Comparator","clockEdge")
  clockEdge:setGateMode()
  local inputEdge = self:createObject("Comparator","inputEdge")
  inputEdge:setGateMode()

  connect(self,"In1",inputEdge,"In")
  connect(inputEdge,"Out",quantizer,"In")
  connect(clockEdge,"Out",quantizer,"Clock")
  connect(quantizer,"Out",self,"Out1")

  if channelCount > 1 then
    connect(quantizer,"Out",self,"Out2")
  end

  self:createMonoBranch("clock",clockEdge,"In",clockEdge,"Out")
end

local views = {
  expanded = {"input","clock"},
  collapsed = {},
}

function QuantizeToClock:onLoadViews(objects,branches)
  local controls = {}
  
  controls.input = InputGate {
    button = "input",
    description = "Unit Input",
    comparator = objects.inputEdge,
    readoutUnits = app.unitHertz
  }

  controls.clock = Gate {
    button = "clock",
    description = "Clock",
    branch = branches.clock,
    comparator = objects.clockEdge,
  }

  return controls, views
end

return QuantizeToClock
